------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2003                 www.xilinx.com
--
--  XAPP 229 - Wide Block RAM in VirtexII and Spartan3 devices
--
------------------------------------------------------------------------------
--
--  File name :       ramb16_s144.vhd
--
--  Description :     Wide (128 x 144) single-port memory in VirtexII
--
--  Date - revision : June 2nd 2006 - v 1.1
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity ramb16_s144 is port (
	di    	 	: in std_logic_vector (127 downto 0);	-- data in
	dip    		: in std_logic_vector (15 downto 0);	-- parity in
	en   		: in std_logic;				-- EN in
	we    		: in std_logic;				-- write enable in
	ssr   		: in std_logic;				-- sync set/reset in
	clk 		: in std_logic;				-- clock
	clkx2 		: in std_logic;				-- clock x2
	rst		: in std_logic;				-- reset
	addr  		: in std_logic_vector (6 downto 0);	-- address in
	do    		: out std_logic_vector (127 downto 0);	-- data out
	dop    		: out std_logic_vector (15 downto 0));	-- parity out
end ramb16_s144;

architecture arch_ramb16_s144 of ramb16_s144 is

signal high 		: std_logic ;
signal low 		: std_logic ;
signal data_muxa 	: std_logic_vector (31 downto 0);
signal data_muxb 	: std_logic_vector (31 downto 0);
signal doa_int 		: std_logic_vector (31 downto 0);
signal dob_int 		: std_logic_vector (31 downto 0);
signal dopa_int 	: std_logic_vector (3 downto 0);
signal dopb_int 	: std_logic_vector (3 downto 0);
signal par_muxa 	: std_logic_vector (3 downto 0);
signal par_muxb		: std_logic_vector (3 downto 0);
signal addr_muxa 	: std_logic_vector (8 downto 0);
signal addr_muxb 	: std_logic_vector (8 downto 0);
signal mux 		: std_logic;
signal en_del		: std_logic;
signal clk_lac		: std_logic;
signal notclk		: std_logic;

begin

low <= '0' ;
high <= '1' ;
notclk <= not clk after 100 pS ;	-- solves a problem with simulation from MTI 6.0 on

-- Generate logic available clock
clkac_lacff : fdc port map (c => clkx2, d => notclk, clr => rst, q => clk_lac);

block_ram : ramb16_s36_s36 port map (
	DIA    		=> data_muxa ,
	DIB    		=> data_muxb,
	DIPA    	=> par_muxa,
	DIPB    	=> par_muxb,
	ENA    		=> en,
	ENB    		=> en,
	WEA    		=> we,
	WEB    		=> we,
	SSRA   		=> ssr,
	SSRB   		=> ssr,
	clkA   		=> clkx2,
	clkB   		=> clkx2,
	ADDRA  		=> addr_muxa,
	ADDRB  		=> addr_muxb,
	DOA    		=> doa_int,
	DOB    		=> dob_int,
	DOPA    	=> dopa_int,
	DOPB    	=> dopb_int);

data_muxa <= di(63 downto 32) when mux = '0' else di(31 downto 0) ;
data_muxb <= di(127 downto 96) when mux = '0' else di(95 downto 64) ;

par_muxa <= dip(7 downto 4) when mux = '0' else dip(3 downto 0) ;
par_muxb <= dip(15 downto 12) when mux = '0' else dip(11 downto 8) ;

addr_muxa <= ("00" & addr) when mux = '0' else ("01" & addr) ;
addr_muxb <= ("10" & addr) when mux = '0' else ("11" & addr) ;

mux <= clk_lac ;
		
process (clk, rst)
begin
if rst = '1' then
	do(31 downto 0) <= (others => '0') ;
	do(95 downto 64) <= (others => '0') ;
	dop(3 downto 0) <= (others => '0') ;
	dop(11 downto 8) <= (others => '0') ;
elsif clk'event and clk = '1' then
	if en = '1' then
		do(31 downto 0) <= doa_int ;
		dop(3 downto 0) <= dopa_int ;
		do(95 downto 64) <= dob_int ;
		dop(11 downto 8) <= dopb_int ;
	end if ;
end if ;
end process ;

process (clk, rst)
begin
if rst = '1' then
	do(127 downto 96) <= (others => '0') ;
	do(63 downto 32) <= (others => '0') ;
	dop(15 downto 12) <= (others => '0') ;
	dop(7 downto 4) <= (others => '0') ;
	en_del <= '0' ;
elsif clk'event and clk = '0' then
	en_del <= en ;
	if en_del = '1' then
		do(63 downto 32) <= doa_int ;
		dop(7 downto 4) <= dopa_int ;
		do(127 downto 96) <= dob_int ;
		dop(15 downto 12) <= dopb_int ;
	end if ;
end if ;
end process ;
				
end arch_ramb16_s144;


